import java.io.*;
import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import java.awt.geom.*;

import javax.swing.*;

import com.mindfusion.diagramming.*;

class FlowChartTransferHandler extends TransferHandler{

	private static boolean _boxCreated = false;

	private class FormTransferable implements Transferable{

		FormTransferable(Image image){
			_image = image;
		}

		public Object getTransferData(DataFlavor flavor)
			throws UnsupportedFlavorException
		{

			if (!isDataFlavorSupported(flavor))
				throw new UnsupportedFlavorException(flavor);

			return _image;
		}

		public DataFlavor[] getTransferDataFlavors()
		{

			return new DataFlavor[]
			{
				DataFlavor.imageFlavor,
				DataFlavor.stringFlavor
			};
		}

		public boolean isDataFlavorSupported(DataFlavor flavor)
		{
			return DataFlavor.imageFlavor.equals(flavor) ||
				DataFlavor.stringFlavor.equals(flavor);
		}

		private Image _image;
	}

	protected Transferable createTransferable(JComponent c)
	{
		if (c instanceof ImageBox)
			return new FormTransferable(((ImageBox)c).getImage());
		else
			return new StringSelection(((JList)c).getSelectedValue().toString() );
	}

	public boolean importData(JComponent c, Transferable t)
	{
		Image image;

		if (canImport(c, t.getTransferDataFlavors()))
		{
			// Can only drop on the flow chart control
			if (!(c instanceof DiagramView))
				return false;

			DiagramView flowChart = (DiagramView)c;

			if (t.isDataFlavorSupported(DataFlavor.imageFlavor))
			{
				try
				{
					image = (Image)t.getTransferData(DataFlavor.imageFlavor);

					// Set the component to the new picture
					// todo create a box
					ShapeNode box = flowChart.getDiagram().getFactory().createShapeNode(
							(int)flowChart.getScrollX(),
							(int)flowChart.getScrollY(), 10, 10);
					box.setImageAlign(ImageAlign.Fit);
					box.setTransparent(true);
					box.setRotateImage(true);
					box.setRotateText(true);

					return true;
				}
				catch (UnsupportedFlavorException ufe)
				{
					System.out.println("importData: unsupported data flavor");
				}
				catch (IOException ioe)
				{
					System.out.println("importData: I/O exception");
				}
			}
			else if (t.isDataFlavorSupported(DataFlavor.stringFlavor))
			{
				try
				{
					String value = (String)t.getTransferData(DataFlavor.stringFlavor);

					_boxCreated = true;

				}
				catch (UnsupportedFlavorException ufe)
				{
					System.out.println("importData: unsupported data flavor");
				}
				catch (IOException ioe)
				{
					System.out.println("importData: I/O exception");
				}
			}
		}

		return false;
	}

	public int getSourceActions(JComponent c)
	{
		return COPY;
	}

	protected void exportDone(JComponent c, Transferable data, int action)
	{
	}

	public boolean canImport(JComponent c, DataFlavor[] flavors)
	{
		if (!(c instanceof DiagramView))
			return false;

		for (int i = 0; i < flavors.length; i++)
		{
			if (DataFlavor.imageFlavor.equals(flavors[i]))
				return true;

			if (DataFlavor.stringFlavor.equals(flavors[i]))
				return true;
		}

		return false;
	}

	public static boolean isNodeCreated(){
		return _boxCreated;
	}

	public static void setBoxCreate(boolean is){
		_boxCreated = is;
	}

}