
package com.mindfusion.diagramming.samples.demo;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;


public class AddPatientDialog extends JDialog
{
	/**
	 * Initializes a new instance of the AddPatientDialog class with
	 * the specified parent. The dialog is centered within
	 * the parent.
	 */
	public AddPatientDialog(JFrame parent)
	{
		super(parent, true);

		_accepted = false;

		setTitle("New Patient");
		setSize(385, 232);
		setResizable(false);

		// Center in parent
		Rectangle parentBounds = new Rectangle();
		parent.getBounds(parentBounds);

		setLocation((parentBounds.width - getWidth()) / 2,
			(parentBounds.height - getHeight()) / 2);

		_image = null;

		// Initialize frame
		initializeComponent();
	}

	/**
	 * Initializes child controls.
	 */
	private void initializeComponent()
	{
		setLayout(null);

		//
		// _lblImage
		//
		_lblImage = new JLabel();
		_lblImage.setLocation(7, 14);
		_lblImage.setSize(106, 145);
		_lblImage.setBorder(new LineBorder(Color.BLACK, 1));
		_lblImage.addMouseListener
		(
			new MouseAdapter()
			{
				public void mouseClicked(MouseEvent e)
				{
					AddPatientDialog.this.onImageClicked(e);
				}
			}
		);

		//
		// _lblName
		//
		_lblName = new JLabel();
		_lblName.setText("Name:");
		_lblName.setLocation(120, 14);
		_lblName.setSize(83, 14);

		//
		// _txtName
		//
		_txtName = new JTextField();
		_txtName.setLocation(120, 28);
		_txtName.setSize(253, 20);

		//
		// _lblBirth
		//
		_lblBirth = new JLabel();
		_lblBirth.setText("Date of birth:");
		_lblBirth.setLocation(120, 55);
		_lblBirth.setSize(83, 14);

		//
		// _txtBirth
		//
		_txtBirth = new JTextField();
		_txtBirth.setLocation(120, 69);
		_txtBirth.setSize(253, 20);

		//
		// _lblAddress
		//
		_lblAddress = new JLabel();
		_lblAddress.setText("Address:");
		_lblAddress.setLocation(120, 97);
		_lblAddress.setSize(83, 14);

		//
		// _txtAddress
		_txtAddress = new JTextArea();
		_txtAddress.setLineWrap(true);
		_txtAddress.setLocation(120, 111);
		_txtAddress.setSize(253, 48);

		// _txtAddressScrollPane
		_txtAddressScrollPane = new JScrollPane();
		_txtAddressScrollPane.setViewportView(_txtAddress);
		_txtAddressScrollPane.setLocation(120, 111);
		_txtAddressScrollPane.setSize(253, 48);

		//
		// _btnAdd
		//
		_btnAdd = new JButton();
		_btnAdd.setText("Add");
		_btnAdd.setLocation(112, 173);
		_btnAdd.setSize(80, 20);
		_btnAdd.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					AddPatientDialog.this.onBtnAdd(e);
				}
			}
		);

		//
		// _btnCancel
		//
		_btnCancel = new JButton();
		_btnCancel.setText("Cancel");
		_btnCancel.setLocation(200, 173);
		_btnCancel.setSize(80, 20);
		_btnCancel.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					AddPatientDialog.this.onBtnCancel(e);
				}
			}
		);

		//
		// _fileDialog
		//
		_fileDialog = new JFileChooser();

		Container contents = getContentPane();

		contents.add(_lblImage);
		contents.add(_lblName);
		contents.add(_txtName);
		contents.add(_lblBirth);
		contents.add(_txtBirth);
		contents.add(_lblAddress);
		contents.add(_txtAddressScrollPane);
		contents.add(_btnAdd);
		contents.add(_btnCancel);
	}

	/**
	 * Fired when the image has been clicked.
	 */
	private void onImageClicked(MouseEvent e)
	{
		if (_fileDialog.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
		{
			try
			{
				_image = javax.imageio.ImageIO.read(
					_fileDialog.getSelectedFile());
				_lblImage.setIcon(new ImageIcon(_image));
			}
			catch (java.io.IOException ex)
			{
			}
		}
	}

	/**
	 * Handles the Add button click.
	 */
	private void onBtnAdd(ActionEvent e)
	{
		_accepted = true;

		hide();
	}

	/**
	 * Handles the Cancel button click.
	 */
	private void onBtnCancel(ActionEvent e)
	{
		_accepted = false;

		hide();
	}

	/**
	 * Gets whether the user has confirmed the dialog.
	 */
	public boolean getAccepted() { return _accepted; }

	/**
	 * Gets the name of the patient.
	 */
	public String getPatientName() { return _txtName.getText(); }

	/**
	 * Gets the birth date of the patient.
	 */
	public String getPatientDate() { return _txtBirth.getText(); }

	/**
	 * Gets the address of the patient.
	 */
	public String getPatientAddress() { return _txtAddress.getText(); }

	/**
	 * Gets the image of the patient.
	 */
	public Image getPatientImage() { return _image; }


	private boolean _accepted;

	private JLabel _lblImage;
	private JLabel _lblName;
	private JTextField _txtName;
	private JLabel _lblBirth;
	private JTextField _txtBirth;
	private JLabel _lblAddress;
	private JTextArea _txtAddress;
	private JScrollPane _txtAddressScrollPane;
	private JButton _btnAdd;
	private JButton _btnCancel;

	private JFileChooser _fileDialog;

	private Image _image;
}
