/**
 * Copyright (c) 2013, MindFusion LLC - Bulgaria.
 */

package samples.onewaylayout;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.imageio.*;
import javax.swing.*;

import com.mindfusion.diagramming.*;


public class MainFrame extends JFrame
	implements ActionListener 
{
	public MainFrame()
	{
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setLocationByPlatform(true);
		setSize(800, 600);
		setTitle("JDiagram Sample: One-way Layout");
		try
		{
			ArrayList<Image> images = new ArrayList<Image>();
			images.add(ImageIO.read(App.class.getResourceAsStream("App.png")));
			images.add(ImageIO.read(App.class.getResourceAsStream("App32.png")));
			setIconImages(images);
		}
		catch (Exception ex)
		{
		}

		initializeComponent();

		try
		{
			diagramView.loadFromXml(App.class.getResourceAsStream("Diagram.xml"));
		}
		catch (XmlException ex)
		{
			JOptionPane.showMessageDialog(this, "Failed to load: " + ex.getMessage(),
				"Open document", JOptionPane.ERROR_MESSAGE);
		}

		Brush diagramBrush = diagramView.getDiagram().getEffectiveBackBrush();
		if (diagramBrush instanceof SolidBrush)
			diagramView.setBackground(((SolidBrush)diagramBrush).getColor());
	}

	private void initializeComponent()
	{
		setLayout(new BorderLayout());

		diagramView = new DiagramView();
		pane = new JScrollPane(diagramView);

		panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		horizontalCheck = new JCheckBox("Horizontal");
		panel.add(horizontalCheck);
		roundLinksCheck = new JCheckBox("Round links");
		panel.add(roundLinksCheck);
		arrangeButton = new JButton("Arrange");
		arrangeButton.addActionListener(this);
		panel.add(arrangeButton);

		Container c = getContentPane();
		c.add(pane, BorderLayout.CENTER);
		c.add(panel, BorderLayout.SOUTH);
	}

	public void actionPerformed(ActionEvent e)
	{
		Diagram diagram = diagramView.getDiagram();
		OneWayLayout layout = new OneWayLayout();
		if (horizontalCheck.isSelected())
			layout.setOrientation(com.mindfusion.diagramming.jlayout.Orientation.Horizontal);
		if (roundLinksCheck.isSelected())
			layout.setRoundedLinks(true);
		layout.arrange(diagram);
		diagram.resizeToFitItems(5);
	}


	private static final long serialVersionUID = 1L;

	private JScrollPane pane;
	private DiagramView diagramView;
	private JPanel panel;
	private JCheckBox horizontalCheck;
	private JCheckBox roundLinksCheck;
	private JButton arrangeButton;
}
