
import java.awt.*;
import java.awt.geom.*;

import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.xml.parsers.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.*;

import org.w3c.dom.*;
import org.xml.sax.*;

import com.mindfusion.diagramming.XmlException;
import com.mindfusion.diagramming.XmlReaderException;

class Diagram{
	
	public Diagram(String name, String uniqueName){
		_name = name;
		_diagData = "";
		_id = uniqueName;
		
		_listeners = new EventListenerList();
	}
	
	public boolean Save(String path, String fileName) 
		throws XmlException
	{
		File file = new File(path, _id + ".cdg");

		if(file.exists() == false)
			{
				try{
					boolean success = file.createNewFile();
					if(success == false)
						JOptionPane.showMessageDialog(new JFrame(), "Haven't file");	
				}
				catch(java.io.IOException exp)
				{
					JOptionPane.showMessageDialog(new JFrame(), exp.getMessage());
				}				
			}
				
		
		DocumentBuilder builder = createBuilderWriter();
        Document document = builder.newDocument();
        write(document);
        saveDOM(document, new StreamResult(file));
		
		return true;
	}
	
	private void write(Document document)
    {
    	this.document = document;
    	document.setXmlStandalone(true);
    	
    	Comment comment = document.createComment("JDiagram diagram");
        document.appendChild(comment);
        
        Element diagram = document.createElement("Diagram");
        document.appendChild(diagram);

		// write name
		Element name = createElement(diagram, "Name");
		writeName(name);
		
		//write image
		Element image = createElement(diagram, "Image");
		writeImage(image);
    }
    
	private void writeName(Element element)
	{	
		org.w3c.dom.Text node = document.createTextNode(_name);
        element.appendChild(node);
	}
	
	private void writeImage(Element element)
	{
		org.w3c.dom.Text node = document.createTextNode(_diagData);
        element.appendChild(node);
	}
	
	private void writeText(String text, Element parent, String name)
    {
        Element element = createElement(parent, name);
        org.w3c.dom.Text node = document.createTextNode(text);
        element.appendChild(node);
    }
	
	private DocumentBuilder createBuilderWriter()
        throws XmlException
    {
        try
        {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setValidating(false);
            factory.setNamespaceAware(false);
            return factory.newDocumentBuilder();
        }
        catch (ParserConfigurationException ex)
        {
            throw new XmlException(ex);
        }
    }
	
	private Element createElement(Element parent, String name)
    {
        Element element = document.createElement(name);
        parent.appendChild(element);
        return element;
    }
	
	private void saveDOM(Document document, StreamResult stream)
        throws XmlException
    {
    	try
        {
			DOMSource source = new DOMSource(document);

            // use an XSLT Transformer for output
            TransformerFactory tFactory =
                TransformerFactory.newInstance();

            Transformer transformer = tFactory.newTransformer();
            transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            transformer.transform(source, stream);
            
        }
        catch (TransformerConfigurationException ex)
        {
            throw new XmlException(ex);
        }
        catch (TransformerException ex)
        {
        	JOptionPane.showMessageDialog(new JFrame(), ex.getMessage());
            throw new XmlException(ex);
        }
    }
	
	public boolean Load(String fileName)
		throws XmlReaderException
	{

		File readFile = new File(fileName);
		
		if(!readFile.exists())
			return false;
		
		
		try
        {
            DocumentBuilder builder = createBuilderReader();
            Document document = builder.parse(readFile);
            read(document);
            
            
    	}
        catch (IOException ex)
        {
            throw new XmlReaderException(ex);
        }
        catch (SAXException ex)
        {
            throw new XmlReaderException(ex);
        }
		
		
		return true;
	}
	
	private void read(Document document)
	{
		Element diagram = document.getDocumentElement();
		
		if(diagram != null)
		{
			
			org.w3c.dom.NodeList nodeDiagName = diagram.getElementsByTagName("Name");
			//String diagName = nodeDiag.item(0);
			
			org.w3c.dom.NodeList nodeDiagImage = diagram.getElementsByTagName("Image");
			_diagData = nodeDiagImage.item(0).getTextContent();
			
		}
	}
	
	private Element getChildElement(Element element, String name)
    {
        if (element == null)
            return null;

        org.w3c.dom.NodeList children = element.getChildNodes();
        for (int i = 0; i < children.getLength(); ++i)
        {
            org.w3c.dom.Node node = children.item(i);
            if (node.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE &&
                							node.getNodeName().equals(name))
                return (Element)children.item(i);
        }

        return null;
    }
	
	private DocumentBuilder createBuilderReader()
        throws XmlReaderException
    {
        try
        {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setValidating(false);
            factory.setNamespaceAware(false);
            return factory.newDocumentBuilder();
        }
        catch (ParserConfigurationException ex)
        {
            throw new XmlReaderException(ex);
        }
    }
    
    private void fireChanged()
    {
        // guaranteed to return a non-null array
        Object[] listeners = _listeners.getListenerList();
        ChangeEvent event = null;

        // process the listeners last to first, notifying
        // those that are interested in this event
        for (int i = listeners.length-2; i>=0; i-=2)
        {
            // lazily create the event:
            if (event == null)
                event = new ChangeEvent(this);
            ((ChangeListener)listeners[i+1]).stateChanged(event);
        }
    }
	
    public void addListener(ChangeListener listener)
    {
        _listeners.add(ChangeListener.class, listener);
    }

    public void removeListener(ChangeListener listener)
    {
        _listeners.remove(ChangeListener.class, listener);
    }

    
    
    
	public void setName(String name){
		_name = name;
		
		fireChanged();
	}
	
	public String getName(){
		return _name;
	}
	
	public void setData( String diagData){
		_diagData = diagData;
	}
	
	public String getData(){
		return _diagData;
	}
	
	public String getID(){
		return _id;
	}
	
	
	
	private String _name;
	private String _diagData;
	private String _id;
	
	private Document document;
	private int _yIndent;
	private com.mindfusion.diagramming.Diagram flowChart;
	private EventListenerList _listeners;
}


//
//class Project
//
public class Project{
	
	public Project()
	{
		_diagrams = new ArrayList();
		_filesToDelete = new ArrayList();
	}	
	
	public boolean Create(String folder, String title)
	{
		_folder = folder;
		_title = title;
		_modified = true;

		return true;
	}
		
	public boolean Save()
		throws XmlException
	{
		if (!_modified)
			return true;
	
		// Create project folder
		File _file = new File(_folder);
		
		if(!_file.exists())	
			_file.mkdir();
		
		
		// Delete files of removed diagrams
		File thisFile;
		String s = "";
		for(int i = 0; i < _filesToDelete.size(); i++){
			s = _filesToDelete.get(i).toString();
			thisFile = new File(_title + s + ".cdg");
			thisFile.delete();
		}
		
		
		_filesToDelete.clear();
		
		File projectFile = new File(_folder + "\\" +"Project.cdp");
		
		// Save project
		DocumentBuilder builder = createBuilderWriter();
        Document document = builder.newDocument();
        SaveDiagrams(document);
        saveDOM(document, new StreamResult(projectFile));
				
		return true;
	}


	private boolean SaveDiagrams(Document doc)
		throws XmlException
	{
		this.document = doc;
		Element project = document.createElement("Project");
		project.setAttribute("Version", Integer.toString(FileFormatVersion()));
		document.appendChild(project);

		Element diagrams = createElement(project ,"Diagrams");
		diagrams.setAttribute("Count", Integer.toString(_diagrams.size()));
		project.appendChild(diagrams);
		
		for(int i = 0; i < _diagrams.size(); i++)
		{
			Diagram d = (Diagram)_diagrams.get(i);
			Element element = createElement(diagrams, "Diagram");
			element.setAttribute("Id", d.getID());
			diagrams.appendChild(element);
			
			d.Save(_folder, d.getName());	
		}
		
		
		return true;
	}



	private Element createElement(Element parent, String name)
    {
        Element element = document.createElement(name);
        parent.appendChild(element);
        return element;
    }
	
	
	private void saveDOM(Document document, StreamResult stream)
        throws XmlException
    {
        try
        {
            DOMSource source = new DOMSource(document);

            // use an XSLT Transformer for output
            TransformerFactory tFactory =
                TransformerFactory.newInstance();

            Transformer transformer = tFactory.newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            transformer.transform(source, stream);
        }
        catch (TransformerConfigurationException ex)
        {
            throw new XmlException(ex);
        }
        catch (TransformerException ex)
        {
            throw new XmlException(ex);
        }
    }
    
	private DocumentBuilder createBuilderWriter()
        throws XmlException
    {
        try
        {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setValidating(false);
            factory.setNamespaceAware(false);
            return factory.newDocumentBuilder();
        }
        catch (ParserConfigurationException ex)
        {
            throw new XmlException(ex);
        }
    }
    
    
    
    
	public boolean Load(String fileName)
		throws XmlReaderException
	{
		
		File file = new File(fileName);
		
		if(!file.exists())
			return false;
		
		_folder = file.getParent();
		_filesToDelete.clear();
		
		try
		{
			DocumentBuilder builder = createBuilderReader();
			Document document = builder.parse(file);
			read(document);
		}
		catch(IOException ex)
		{
			throw new XmlReaderException(ex);
		}
		catch(SAXException ex)
		{
			throw new XmlReaderException(ex);	
		}
		
		return true;
	}
	
	private void read(Document document)
	{
			
			
	/*here
	*	if (reader.NodeType == XmlNodeType.XmlDeclaration)
 	*	continue;
 	*
 	*/	
		Element diagram = document.getDocumentElement();
		
		//read diargams
		Element diagramsElement = getChildElement(diagram, "Diagrams");
		if(diagramsElement != null)
		{
			try{
				LoadDiagram(document, diagramsElement);
			}
			catch(XmlReaderException e){}
		}
	}	
	
	private boolean LoadDiagram(Document document, Element el)
		throws XmlReaderException
	{
		
		int ix = 0;
		String attr = el.getAttribute("Count");
		int count = Integer.parseInt(attr);
		int depth = document.getChildNodes().getLength();
		
		_diagrams.clear();
		
		if (count == 0)
			return true;
		
		org.w3c.dom.NodeList nodeDiag = el.getElementsByTagName("Diagram");
		for(int i = 0; i < count; i++)
		{
			Element diagramElement = (Element)(nodeDiag.item(i));
			Diagram diag = new Diagram("", diagramElement.getAttribute("Id"));
			diag.addListener(new ChangeListener(){
				public void stateChanged(ChangeEvent e){
					_modified = true;
				}
			});
			String fileName = _folder + "\\" + diag.getID() + ".cdg"; 
			diag.Load(fileName);
			_diagrams.add(diag);
			ix++;
		}

		return false;
	}
	
	public void AddDiagram(Diagram diagram)
	{
		_diagrams.add(diagram);
		diagram.addListener(new ChangeListener()
		{
			public void stateChanged(ChangeEvent e)
			{
				_modified = true;
			}
		});
		_modified = true;
	}
	
	public void RemoveDiagram(Diagram diagram)
	{
		_diagrams.remove(diagram);
		_filesToDelete.add(diagram.getID());
		_modified = true;
	}
	
	private Element getChildElement(Element element, String name)
    {
        if (element == null)
            return null;

        org.w3c.dom.NodeList children = element.getChildNodes();
        for (int i = 0; i < children.getLength(); ++i)
        {
            org.w3c.dom.Node node = children.item(i);
            if (node.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE &&
                node.getNodeName().equals(name))
                return (Element)children.item(i);
        }

        return null;
    }

	private DocumentBuilder createBuilderReader()
        throws XmlReaderException
    {
        try
        {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setValidating(false);
            factory.setNamespaceAware(false);
            return factory.newDocumentBuilder();
        }
        catch (ParserConfigurationException ex)
        {
            throw new XmlReaderException(ex);
        }
        
    }

    private static int FileFormatVersion(){
    	return 1;
    }
    
    public boolean getIsModified()
    {
    	return _modified;
    }
    
    public String getTitle()
    {
    	return _title;
    }
    
    public void setTitle(String _title)
    {
    	this._title = _title;	
    	_modified = true;
    }
    
    public Diagram[] getDiagrams()
    {
    	Diagram[] result = new Diagram[_diagrams.size()];
    	
    	for(int i = 0; i < result.length; i++)
    	{
    		result[i] = (Diagram)(_diagrams.get(i));
    	}
    	
    	return result;	
    }
    
    
	private boolean _modified;
	private String _folder;
	private String _title;

	private ArrayList _diagrams;
	private ArrayList _filesToDelete;
	private Document document;
	
}